// GlpkSharp.h

#pragma once

extern "C" {
#include "glpk.h"

}

using namespace System;
using namespace System::Runtime::InteropServices;
namespace GlpkSharp {

	public enum class OptimisationDirection{
		MINIMISE=LPX_MIN,
		MAXIMISE=LPX_MAX
	};

	public enum class BOUNDSTYPE{
		Free=LPX_FR,
		Lower=LPX_LO,
		Upper=LPX_UP,
		Double=LPX_DB,
		Fixed=LPX_FX
	};

	public enum class BASESTATUS{
		Basic=LPX_BS,
		NonBasicLower=LPX_NL,
		NonBasicUpper=LPX_NU,
		NonBasicFree=LPX_NF,
		NonBasicFixed=LPX_NS
	};

	public enum class SOLVERSTATUS{
		Solved=LPX_E_OK,
		FaultyMatrix=LPX_E_FAULT,
		LowerBound=LPX_E_OBJLL,
		UpperBound=LPX_E_OBJUL,
		IterationLimit=LPX_E_ITLIM,
		TimeLimit=LPX_E_TMLIM,
		SolverFailure=LPX_E_SING,
		NoPrimalFeasible=LPX_E_NOPFS,
		NoDualFeasible=LPX_E_NODFS
	};
	public enum class SOLUTIONSTATUS{
		Optimal=LPX_OPT,
		Feasible=LPX_FEAS,
		Infeasible=LPX_INFEAS,
		NotFeasible=LPX_NOFEAS,
		Unbounded=LPX_UNBND,
		Undefined=LPX_UNDEF
	};
	public enum class WARMUP{
		OK=LPX_E_OK,
		EMPTY=LPX_E_EMPTY,
		BadBasis=LPX_E_BADB,
		Singular=LPX_E_SING
	};

	public enum class INTERIORSTATUS{
		Optimal=LPX_E_OK,
		Faulty=LPX_E_FAULT,
		NotFeasible=LPX_E_NOFEAS,
		NoConvergence=LPX_E_NOCONV,
		IterationLimit=LPX_E_ITLIM,
		Instable=LPX_E_INSTAB
	};
	public enum class MODELCLASS{
		LP=LPX_LP,
		MIP=LPX_MIP
	};
	public enum class COLKIND{
		Continuous=LPX_CV,
		Integer=LPX_IV
	};
	public enum class MIPSTATUS{
		Undefined=LPX_I_UNDEF,
		Optimal=LPX_I_OPT,
		Feasible=LPX_I_FEAS,
		NotFeasible=LPX_I_NOFEAS
	};

	public enum class INTEGERPARAM{
		MessageLevel=LPX_K_MSGLEV,
		Scale=LPX_K_SCALE,
		Dual=LPX_K_DUAL,
		Price=LPX_K_PRICE,
		Round=LPX_K_ROUND,
		IterationLimit=LPX_K_ITLIM,
		ITCNT=LPX_K_ITCNT,
		OUTFRQ=LPX_K_OUTFRQ,
		BRANCH=LPX_K_BRANCH,
		BTRACK=LPX_K_BTRACK,
		MPSINFO=LPX_K_MPSINFO,
		MPSOBJ=LPX_K_MPSOBJ,
		MPSORIG=LPX_K_MPSORIG,
		MPSWIDE=LPX_K_MPSWIDE,
		MPSFREE=LPX_K_MPSFREE,
		MPSSKIP=LPX_K_MPSSKIP,
		PRESOL=LPX_K_PRESOL,
		USECUTS=LPX_K_USECUTS
	};
	public enum class REALPARAM{
		Relax=LPX_K_RELAX,
		Tolerance=LPX_K_TOLBND,
		Toldj=LPX_K_TOLDJ,
		Tolpiv=LPX_K_TOLPIV,
		Objll=LPX_K_OBJLL,
		Objul=LPX_K_OBJUL,
		TMLIM=LPX_K_TMLIM,
		OutDelay=LPX_K_OUTDLY,
		TOLINT=LPX_K_TOLINT,
		TOLOBJ=LPX_K_TOLOBJ,
	};
	public enum class CUTS{
		COVER=LPX_C_COVER,
		CLIQUE=LPX_C_CLIQUE,
		GOMORY=LPX_C_GOMORY,
		ALL=LPX_C_ALL
	};

	public ref class LPProblem
	{
		// TODO: Add your methods for this class here.
	private: LPX* prob;
		LPProblem(int empty){

		}
	public:
		LPProblem(){
			prob=lpx_create_prob();
		}


		~LPProblem(){
			lpx_delete_prob(prob);
		}

		!LPProblem(){
			lpx_delete_prob(prob);
		}

		property String^ Name{
			String^ get(){
				return gcnew String(lpx_get_prob_name(prob));
			}
			void set(String^ s){
				char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(s).ToPointer());
				lpx_set_prob_name(prob,sp);
				Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			}
		}

		property String^ ObjectiveName{
			String^ get(){
				return gcnew String(lpx_get_obj_name(prob));
			}
			void set(String^ s){
				char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(s).ToPointer());
				lpx_set_obj_name(prob,sp);
				Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			}
		}

		property OptimisationDirection ObjectiveDirection{
			OptimisationDirection get(){
				return OptimisationDirection( lpx_get_obj_dir(prob));
			}
			void set(OptimisationDirection dir){
				lpx_set_obj_dir(prob,int(dir));
			}
		}

		/*
Add rows to the problem and return the index of the first row added.

*/
		int AddRows(int number){
			return lpx_add_rows(prob,number);
		}
		void DeleteRows(array<int>^ rowsId){
			pin_ptr<int> pint=&rowsId[0];
			lpx_del_rows(prob,rowsId->Length-1,pint);
		}
		int GetRowsCount(){
			return lpx_get_num_rows(prob);
		}

		int AddCols(int number){
			return lpx_add_cols(prob,number);
		}
		void DeleteCols(array<int>^ colsId){
			pin_ptr<int> pint=&colsId[0];
			lpx_del_cols(prob,colsId->Length-1,pint);
		}
		int GetColsCount(){
			return lpx_get_num_cols(prob);
		}

		void SetRowName(int i,String^ name){
				char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(name).ToPointer());
				lpx_set_row_name(prob,i,sp);
				Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
		}

		String^ GetRowName(int i){
			return gcnew String(lpx_get_row_name(prob,i));
		}

		void SetColName(int i,String^ name){
				char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(name).ToPointer());
				lpx_set_col_name(prob,i,sp);
				Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
		}

		String^ GetColName(int i){
			return gcnew String(lpx_get_col_name(prob,i));
		}

		void SetRowBounds(int i,BOUNDSTYPE bt, double lb, double ub){
			lpx_set_row_bnds(prob,i,int(bt),lb,ub);
		}

		BOUNDSTYPE GetRowBoundType(int i){
			return BOUNDSTYPE(lpx_get_row_type(prob,i));
		}
		double GetRowLb(int i){
			return lpx_get_row_lb(prob,i);
		}
		double GetRowUb(int i){
			return lpx_get_row_ub(prob,i);
		}

		void SetColBounds(int i,BOUNDSTYPE bt, double lb, double ub){
			lpx_set_col_bnds(prob,i,int(bt),lb,ub);
		}
		BOUNDSTYPE GetColBoundType(int i){
			return BOUNDSTYPE(lpx_get_col_type(prob,i));
		}
		double GetColLb(int i){
			return lpx_get_col_lb(prob,i);
		}
		double GetColUb(int i){
			return lpx_get_col_ub(prob,i);
		}

		void SetObjCoef(int i, double coeff){
			lpx_set_obj_coef(prob,i,coeff);
		}
		double GetObjCoef(int i){
			return lpx_get_obj_coef(prob,i);
		}

		void SetMatRow(int i, array<int>^ ind, array<double>^ val){
			pin_ptr<int> pint=&ind[0];
			pin_ptr<double> pdouble=&val[0];
			lpx_set_mat_row(prob,i,ind->Length-1,pint,pdouble);
		}
		int GetMatRow(int i,[Out]array<int>^ %ind,[Out]array<double>^ %val){
			//int pind[];
			//double pval[];
			int numcols=GetColsCount();
			int* pind=new int[numcols+1];
			double* pval=new double[numcols+1];
			int num=lpx_get_mat_row(prob,i,pind,pval);
			ind=gcnew array<int>(num+1);
			val=gcnew array<double>(num+1);
			//Marshal::Copy(&pind[0],ind,0,num+1);
			//Marshal::Copy(&pval[0],val,0,num+1);
			IntPtr ipind=*new IntPtr(pind);
			IntPtr ipval=*new IntPtr(pval);
			Marshal::Copy(ipind,ind,(int)0,num+1);
			Marshal::Copy(ipval,val,(int)0,num+1);
			delete(pind);
			delete(pval);
			return num;
		}

		void SetMatCol(int i, array<int>^ ind, array<double>^ val){
			pin_ptr<int> pint=&ind[0];
			pin_ptr<double> pdouble=&val[0];
			lpx_set_mat_col(prob,i,ind->Length-1,pint,pdouble);
		}
		int GetMatCol(int i,[Out]array<int>^ %ind,[Out]array<double>^ %val){
			//int pind[];
			//double pval[];
			int numrows=GetRowsCount();
			int* pind=new int[numrows+1];
			double* pval=new double[numrows+1];
			int num=lpx_get_mat_col(prob,i,pind,pval);
			ind=gcnew array<int>(num+1);
			val=gcnew array<double>(num+1);
			//Marshal::Copy(&pind[0],ind,0,num+1);
			//Marshal::Copy(&pval[0],val,0,num+1);
			IntPtr ipind=*new IntPtr(pind);
			IntPtr ipval=*new IntPtr(pval);
			Marshal::Copy(ipind,ind,(int)0,num+1);
			Marshal::Copy(ipval,val,(int)0,num+1);
			delete(pind);
			delete(pval);
			return num;
		}

		void LoadMatrix(array<int>^ rind,array<int>^ cind, array<double>^ val){
			//if(rind->Length!=cind->Length){
			//	return;
			//}
			pin_ptr<int> rpint=&rind[0];
			pin_ptr<int> cpint=&cind[0];
			pin_ptr<double> pdouble=&val[0];
			lpx_load_matrix(prob,rind->Length-1,rpint,cpint,pdouble);
		}

		int GetNonZeroCount(){
			return lpx_get_num_nz(prob);
		}

		void ScaleProb(){
			lpx_scale_prob(prob);
		}
		void UnscaleProb(){
			lpx_unscale_prob(prob);
		}
		void BuildStdBasis(){
			lpx_std_basis(prob);
		}
		void BuildAdvBasis(){
			lpx_adv_basis(prob);
		}
		void SetRowStatus(int i, BASESTATUS s){
			lpx_set_row_stat(prob,i,(int)s);
		}
		void SetColStatus(int i, BASESTATUS s){
			lpx_set_col_stat(prob,i,(int)s);
		}
		SOLVERSTATUS SolveSimplex(){
			return SOLVERSTATUS(lpx_simplex(prob));
		}
		SOLUTIONSTATUS GetStatus(){
			return SOLUTIONSTATUS(lpx_get_status(prob));
		}
		SOLUTIONSTATUS GetPrimalStatus(){
			return SOLUTIONSTATUS(lpx_get_prim_stat(prob));
		}
		SOLUTIONSTATUS GetDualStatus(){
			return SOLUTIONSTATUS(lpx_get_dual_stat(prob));
		}

		double GetObjectiveValue(){
			return lpx_get_obj_val(prob);
		}

		BASESTATUS GetRowStatus(int i){
			return BASESTATUS(lpx_get_row_stat(prob,i));
		}
		double GetRowPrimal(int i){
			return lpx_get_row_prim(prob,i);
		}
		double GetRowDual(int i){
			return lpx_get_row_dual(prob,i);
		}

		BASESTATUS GetColStatus(int i){
			return BASESTATUS(lpx_get_col_stat(prob,i));
		}
		double GetColPrimal(int i){
			return lpx_get_col_prim(prob,i);
		}
		double GetColDual(int i){
			return lpx_get_col_dual(prob,i);
		}
		int GetUnboundedVariableIndex(){
			return lpx_get_ray_info(prob);
		}

		WARMUP WarmUp(){
			return WARMUP(lpx_warm_up(prob));
		}

		int EvalTabRow(int i,array<int>^ ind,array<double>^ val){
			int numcols=GetColsCount();
			int* pind=new int[numcols+1];
			double* pval=new double[numcols+1];
			int num=lpx_eval_tab_row(prob,i,pind,pval);
			ind=gcnew array<int>(num+1);
			val=gcnew array<double>(num+1);
			IntPtr ipind=*new IntPtr(pind);
			IntPtr ipval=*new IntPtr(pval);
			Marshal::Copy(ipind,ind,(int)0,num+1);
			Marshal::Copy(ipval,val,(int)0,num+1);
			delete(pind);
			delete(pval);
			return num;
		}

		int EvalTabCol(int i,array<int>^ ind,array<double>^ val){
			int numcols=GetRowsCount();
			int* pind=new int[numcols+1];
			double* pval=new double[numcols+1];
			int num=lpx_eval_tab_col(prob,i,pind,pval);
			ind=gcnew array<int>(num+1);
			val=gcnew array<double>(num+1);
			IntPtr ipind=*new IntPtr(pind);
			IntPtr ipval=*new IntPtr(pval);
			Marshal::Copy(ipind,ind,(int)0,num+1);
			Marshal::Copy(ipval,val,(int)0,num+1);
			delete(pind);
			delete(pval);
			return num;
		}

		INTERIORSTATUS InteriorPoint(){
			return INTERIORSTATUS(lpx_interior(prob));
		}

		double IPGetObjectiveValue(){
			return lpx_ipt_obj_val(prob);
		}
		double IPGetRowPrimal(int i){
			return lpx_ipt_row_prim(prob,i);
		}
		double IPGetRowDual(int i){
			return lpx_ipt_row_dual(prob,i);
		}
		double IPGetColPrimal(int i){
			return lpx_ipt_col_prim(prob,i);
		}
		double IPGetColDual(int i){
			return lpx_ipt_col_dual(prob,i);
		}

		property MODELCLASS ModelClass{
			MODELCLASS get(){
				return MODELCLASS(lpx_get_class(prob));
			}
			void set(MODELCLASS c){
				lpx_set_class(prob,(int)c);
			}
		}
		
		void SetColKind(int i, COLKIND k){
			lpx_set_col_kind(prob,i,(int)k);
		}
		COLKIND GetColKind(int i){
			return COLKIND(lpx_get_col_kind(prob, i));
		}
		int GetIntegerColCount(){
			return lpx_get_num_int(prob);
		}
		int GetBinaryColCount(){
			return lpx_get_num_bin(prob);
		}

		SOLVERSTATUS SolveInteger(){
			return SOLVERSTATUS(lpx_integer(prob));
		}
		SOLVERSTATUS SolveIntegerAdvanced(){
			return SOLVERSTATUS(lpx_intopt(prob));
		}
		MIPSTATUS GetMIPStatus(){
			return MIPSTATUS(lpx_mip_status(prob));
		}
		double GetMIPObjectiveValue(){
			return lpx_mip_obj_val(prob);
		}
		double GetMIPRowVal(int i){
			return lpx_mip_row_val(prob,i);
		}
		double GetMIPColVal(int i){
			return lpx_mip_col_val(prob,i);
		}

		void ResetParams(){
			lpx_reset_parms(prob);
		}
		void SetIntControlParam(int p, int val){
			lpx_set_int_parm(prob,p,val);
		}
		int GetIntControlParam(int p){
			return lpx_get_int_parm(prob,p);
		}
		void SetRealControlParam(int p, double val){
			lpx_set_real_parm(prob,p,val);
		}
		double GetRealControlParam(int p){
			return lpx_get_real_parm(prob,p);
		}

		static LPProblem^ ReadMPS(String^ fname){
			LPProblem^ p=gcnew LPProblem(1);
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			p->prob=lpx_read_mps(sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return p;
		}

		int WriteMPS(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_write_mps(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int ReadLPBasis(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_read_bas(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int WriteLPBasis(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_write_bas(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		static LPProblem^ ReadFreeMPS(String^ fname){
			LPProblem^ p=gcnew LPProblem(1);
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			p->prob=lpx_read_freemps(sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return p;
		}

		int WriteFreeMPS(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_write_freemps(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		static LPProblem^ ReadCPLEX(String^ fname){
			LPProblem^ p=gcnew LPProblem(1);
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			p->prob=lpx_read_cpxlp(sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return p;
		}

		int WriteCPLEX(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_write_cpxlp(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		static LPProblem^ ReadModel(String^ model,String^ data, String^ output){
			LPProblem^ p=gcnew LPProblem(1);
			char * spmodel=static_cast<char *>(Marshal::StringToHGlobalAnsi(model).ToPointer());
			char * spdata=static_cast<char *>(Marshal::StringToHGlobalAnsi(data).ToPointer());
			char * spoutput=static_cast<char *>(Marshal::StringToHGlobalAnsi(output).ToPointer());
			p->prob=lpx_read_model(spmodel,spdata,spoutput);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(spmodel));
			Marshal::FreeHGlobal(safe_cast<IntPtr>(spdata));
			Marshal::FreeHGlobal(safe_cast<IntPtr>(spoutput));
			return p;
		}
		int WriteProb(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_print_prob(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int WriteSol(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_print_sol(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int WriteBoundsSensitivity(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_print_sens_bnds(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int WriteIPS(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_print_ips(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}
		int WriteMIP(String^ fname){
			char * sp=static_cast<char *>(Marshal::StringToHGlobalAnsi(fname).ToPointer());
			int i=lpx_print_mip(prob,sp);
			Marshal::FreeHGlobal(safe_cast<IntPtr>(sp));
			return i;
		}



	};
}
